!
! calcHamMat --- build the Hamiltonian matrix
! Robin Santra, Christian Buth
! 23 May 2006
!
!
! calcHamMat --- assemble non-CAP part of the Hamiltonian
!
! Hmat : complex entry matrix storage for the Hamiltonian
!

      subroutine calcHamMat(dimen,nphmin,nphmax,lmin,lmax,nvec, &
     &     mi,cconst,Hmat,eval,Eground,omega,SESdipatm)
!      ,         &
!      &     nstart, nocc, indstart)
      use floquetmat
      implicit none

      integer, intent(in) :: dimen, nphmin, nphmax, lmin, lmax, nvec, mi
!       , nstart
!       integer, optional, intent(in) :: nocc(lmin:lmax)
!       integer, optional, intent(out) :: indstart(nstart)

      real(dblprec), intent(in) :: cconst             ! Coupling constant sqrt(2 pi alpha intensity)
      real(dblprec), intent(in) :: eval(nvec,lmin:lmax)
      real(dblprec), intent(in) ::  Eground, omega
!       real(dblprec) Datom(nvec,lmin:lmax,nvec,lmin:lmax)
      complex(dblprec), intent(in) :: SESdipatm(nvec,lmin:lmax,nvec,lmin:lmax)
      complex(dblprec), intent(out) :: Hmat(dimen,dimen)

      real(dblprec) scal
!       real(dblprec) spectrum(dimen)
!       real(dblprec) work(30*dimen)
!       integer li                ! ,init
!       integer seldim
!       real(dblprec) svec(nvec,lmin:lmax,nphmin:nphmax,seldim)
!       real(dblprec) ovl(seldim)
      integer indr, &                   ! Hamiltonian matrix row index
     &        indc, &                   ! Hamiltonian matrix column index
     &        mur,  &                   ! mu row index
     &        muc                       ! mu column index
      integer lr,lc
      integer i, j
!       , iocc
!       integer info
!       integer idx
!       integer idamax
!       real(dblprec) Hmat(dimen,dimen)

!       write (*,*) 'Eground = ', Eground

      ! This calculates the upper triangle of the Hamiltonian matrix
!       write (*,*) 'Row and column indices of the occupied space:'
!       iocc = 0                          ! Start to search for the first occupied
!                                         ! orbital index
      indc = 0                          ! Column index of Hamiltonian
      do muc = nphmin, nphmax
        do lc = lmin, lmax
          do j = 1, nvec
            indc = indc + 1
            indr = 0                    ! Row index of Hamiltonian

            ! There is only coupling for mur = muc or muc +-1
            do mur = nphmin, muc-2
              do lr = lmin, lmax
                do i = 1, nvec
                  indr = indr + 1
                  Hmat(indr,indc) = 0
                end do
              end do
            end do

            ! Do coupling blocks; this is mur = muc-1
            if (muc .gt. nphmin) then
              mur = muc - 1
              do lr = lmin, lmax
                call calcscal(lr,lc,mi,scal)
                scal = scal * cconst
                do i = 1, nvec
                  indr = indr + 1
!                   Hmat(indr,indc) = scal * Datom(i,lr,j,lc)
                  Hmat(indr,indc) = cmplx(scal, kind=dblprec) * SESdipatm(i,lr,j,lc)
                end do
              end do
            end if

            ! This is the diagonal mur = muc
            mur = muc
            do lr = lmin, lc-1
              do i = 1, nvec
                indr = indr + 1
                Hmat(indr,indc) = 0
              end do
            end do
            ! The diagonal mur = muc and lr = lc
            lr = lc
            do i = 1, j-1
              indr = indr + 1
              Hmat(indr,indc) = 0
            end do
            ! The full diagonal mur = muc, lr = lc and i = j
            i = j
            indr = indr + 1
            ! Here indr == indc holds
            Hmat(indr,indc) = cmplx(eval(j,lc) - Eground  &
     &                              - real(muc, dblprec) * omega, kind=dblprec)
!             ! Identify indices of start vectors for the band-Lanczos:
!             ! choose from the zero photon block mur == muc == 0 the
!             ! occupuied one-particle states
!             if (present(nocc)) then
!               if (muc == 0 .and. i <= nocc(lc)) then
!                 if (present(indstart)) then
!                   iocc = iocc + 1
!                   indstart(iocc) = indr
!                 end if
!               end if
!             end if
          end do
        end do
      end do

      ! Copy the upper triangle into the lower triangle of the matrix
      do indc=1, dimen
        do indr=1, indc-1
          Hmat(indc,indr) = Hmat(indr,indc)
        end do
      end do


      contains

!
! calcscal --- calculate the Clebsch-Gordon coefficients for the
!       dipole operator
!
! lf : final angular momentum
! li : initial angular momentum
! mi : initial magnetic quantum number
! scal : return value
!

      subroutine calcscal(lf,li,mi,scal)
      use anglib
      implicit none

      integer lf,li,mi
      real(dblprec) scal

      scal = dsqrt(dble(2*li+1)/dble(2*lf+1))
      scal = scal*cleb(2*li,2*mi,2,0,2*lf,2*mi)
      scal = scal*cleb(2*li,0,2,0,2*lf,0)
      end subroutine calcscal
      end subroutine calcHamMat
