!
! mtrxbuild --- Calculate the matrix representation in eigenfunctions of the
!       Hamiltonian of various operators
! 19 May 2006
! Robin Santra, Christian Buth
!

      subroutine mtrxbuild(Cevec, nbasis, nstate, nvlnc, ntot, nl, xnd, Eng,  &
     &          ndcntfin, ngrd, egfxn, egder, xh, ionr, lastpotentialvalue, ipottype)
      use FiniteElements
      use SESpath
      implicit none

      integer nbasis, nstate, nvlnc, ntot, nl, ngrd, ndcntfin(nstate,nl), ipottype
      real(dblprec) Cevec(nbasis,nstate,nl), Eng(nstate,nl)
      real(dblprec) egfxn(ngrd,nstate,nl), egder(ngrd,nstate,nl)
      real(dblprec) xh, ionr, lastpotentialvalue
      type(FEBnodes) xnd
!       REAL(dblprec), ALLOCATABLE :: Dple(:,:), Dpbig(:,:)
!       REAL(dblprec), ALLOCATABLE :: Wbig(:,:,:)
      complex(dblprec), allocatable :: SESdip(:,:,:,:), SESfTf(:,:,:), &
     &                                SESpot(:,:,:), SESWbig(:,:,:)
!       REAL(dblprec), ALLOCATABLE :: CoulInt(:)
      integer it0, it1, ict

      call system_clock( it0, ict )
      write (*,*) 'Building matrices using eigenvectors'

      ! Create dipole matrix (analytic)
!       allocate(Dple(-5:5,nbasis), Dpbig(ntot,ntot))
!       call Dmtrx(Dple, xnd)
!       call symtrxcalc(Cevec, nbasis, ntot, Dple, Dpbig)

      ! Create dipole matrix (numerical)
      ! Note that dip is diagonal with respect to l
      write (*,*) '    SES dipole matrix'
      allocate(SESdip(nstate,nl,nstate,nl))
      call SESDmtrx(SESdip, nstate, nl, egfxn, ngrd, xh)

      ! Create W matrix (numerical)
      ! Note that W is diagonal with respect to l
!       write (*,*) '    Riss-Meyer CAP matrix'
!       allocate(Wbig(nstate,nstate,nl))
!       call Wmtrx(Wbig, nstate, nl, egfxn, ngrd, xh, c)

      ! Create transformed kinetic energy matrix (numerical)
      ! Note that fTf is diagonal with respect to l
      write (*,*) '    SES transformed kinetic energy matrix'
      allocate(SESfTf(nstate,nstate,nl))
      call fTfmtrx(SESfTf, nstate, nl, egfxn, egder, ngrd, xh)

      ! Create the SES potential energy correction (numerical)
      ! Note that fTf is diagonal with respect to l
      write (*,*) '    SES potential energy matrix'
      if (ionr > sesr0) then
        write (*,*) 'The atomic potential is not yet the one of a point ', &
     &              'charge when the SES CAP starts to take effect!'
!         stop 'ionr <= r0'
      end if
      allocate(SESpot(nstate,nstate,nl))
      call SESpotmtrx(SESpot, nstate, nl, egfxn, ngrd, xh, lastpotentialvalue, ipottype)

      ! Create W_SES matrix (numerical)
      ! Note that W_SES is diagonal with respect to l
      write (*,*) '    SES CAP matrix'
      allocate(SESWbig(nstate,nstate,nl))
      call SESWmtrx(SESWbig, nstate, nl, egfxn, ngrd, xh)

!! Calculate selected two-electron Coulomb matrix elements

!    ALLOCATE( CoulInt(nstate) )
!       write (*,*) '    Selected two-electron Coulomb matrix elements'
!    CALL twoelectron( nstate, nl, ngrd, egfxn, xh, CoulInt)

      write (*,*) 'Done'

      call system_clock(it1)
      write (*,*) 'Time is', ( (it1-it0)*1.d0/ict )

      call system_clock( it0, ict )

      write (*,*) 'Writing'
      call writeatom(nstate, nvlnc, nl, Eng, ndcntfin, SESdip, SESfTf,  &
     &          SESpot, SESWbig)
      write (*,*) 'Done'

      call system_clock(it1)
      write (*,*) 'Time is', ( (it1-it0)*1.d0/ict )
      deallocate(SESWbig, SESpot, SESfTf)
      end subroutine mtrxbuild
